// ======================================================================
//
// Profile.h
// jeff grills
//
// copyright 1997 origin systems
// 
// ======================================================================

#ifndef PROFILE_H
#define PROFILE_H

#if USE_PROFILE

// ======================================================================

#include "ProfileTime.h"

// ======================================================================

class Profile
{
private:
	
	// prevent objects of this class from being created
	Profile(void);
	Profile(const Profile &);
	Profile &operator =(const Profile &);

friend struct GL_init;

private:

	enum
	{
		MAX_CUMMULATIVE = 8,
		MAX_ENTRIES     = 1024,
		MAX_EXPANDED    = 32,
		MAX_AVERAGE     = 32,
		BUFFER_SIZE     = 25 * 80,
		BUFFER_LIMIT    = BUFFER_SIZE - 80
	};

	struct ProfileEntry
	{
		ProfileTime  time;
		const char  *text;
		s_byte       op;
	};

	struct ProfileCumm
	{
		ProfileTime  time;
		const char  *text;
		int          calls;
	};

private:

#if COMPILING_DLL

	static int          *indx;
	static ProfileEntry *entry;

	static const char * const startError;
	static const char * const stopError;
	static const char * const contError;

#else

	static int          indxData;

	static int         *indx;
	static ProfileEntry entry[MAX_ENTRIES];
	static ProfileCumm  cumm[MAX_CUMMULATIVE];

	static int          selectedLine;
	static const char  *base;

	static const char  *expand[MAX_EXPANDED];
	static int          expandCnt;

	static int          hideCummulative;

	static const char  *averageElem;
	static int          averageSum;
	static int          averageArray[MAX_AVERAGE];

	static char         buffer[BUFFER_SIZE];

	static const char * const startError;
	static const char * const stopError;
	static const char * const contError;

#endif

private:

	static void dump(void);
	static void reset(void);

public:

#if !COMPILING_DLL

	static void update(void);
	static void draw(void);

	static int  cummulativeHandle(const char *str);
	static void start(int cummHandle);
	static void stop(int cummHandle);

	static void getDllEntries(int **newIndx, ProfileEntry **newEntry);

#else

	static void setDllEntries(int *newIndx,  ProfileEntry *newEntry);

#endif

	static void start(const char *str);
	static void stop(const char *str);
	static void cont(const char *str);
};

// ======================================================================

#if !COMPILING_DLL

inline void Profile::start(int handle)
{
	ProfileTime t;
	getProfileTime(&t);
	cumm[handle].time -= t;
	cumm[handle].calls++;
}

// ----------------------------------------------------------------------

inline void Profile::stop(int handle)
{
	ProfileTime t;
	getProfileTime(&t);
	cumm[handle].time += t;
}

// ----------------------------------------------------------------------

inline void Profile::getDllEntries(int **newIndx, ProfileEntry **newEntry)
{
	*newIndx  = indx;
	*newEntry = entry;
}

#else

// ----------------------------------------------------------------------

inline void Profile::setDllEntries(int *newIndx,  ProfileEntry *newEntry)
{
	indx  = newIndx;
	entry = newEntry;
}

#endif

// ----------------------------------------------------------------------

inline void Profile::start(const char *t)
{
	if (indx)
	{
		DEBUG_FATAL(*indx >= MAX_ENTRIES, (startError));
		getProfileTime(&entry[*indx].time);
		entry[*indx].text = t;
		entry[(*indx)++].op = 1;
	}
}

// ----------------------------------------------------------------------

inline void Profile::stop(const char *t)
{
	if (indx)
	{
		DEBUG_FATAL(*indx >= MAX_ENTRIES, (stopError));
		getProfileTime(&entry[*indx].time);
		entry[*indx].text = t;
		entry[(*indx)++].op = -1;
	}
}

// ----------------------------------------------------------------------

inline void Profile::cont(const char *t)
{
	if (indx)
	{
		DEBUG_FATAL(*indx >= MAX_ENTRIES, (startError));
		entry[*indx].time = entry[*indx-1].time;
		entry[*indx].text = t;
		entry[(*indx)++].op = 1;
	}
}

// ----------------------------------------------------------------------

#define PROFILE_GET_DLL(a,b)   Profile::getDllEntries(a,b)
#define PROFILE_SET_DLL(a,b)   Profile::setDllEntries(a,b)
#define PROFILE_CUMMULATIVE(a) Profile::cummulativeHandle(a)
#define PROFILE_START(a)       Profile::start(a)
#define PROFILE_STOP(a)        Profile::stop(a)
#define PROFILE_UPDATE()       Profile::update()
#define PROFILE_DRAW()         Profile::draw()
#define PROFILE_CALL(a,b)      Profile::start(b), (a), Profile::stop(b)
#define PROFILE_NEXT(a,b)      Profile::cont(b), (a), Profile::stop(b)

// ======================================================================
#else

// no profiling

class Profile
{
public :

	struct ProfileEntry
	{
	};
};

#define PROFILE_GET_DLL(a,b)   ((void)0)
#define PROFILE_SET_DLL(a,b)   ((void)0)
#define PROFILE_CUMMULATIVE(a) 0
#define PROFILE_START(a)       ((void)0)
#define PROFILE_STOP(a)        ((void)0)
#define PROFILE_UPDATE()       ((void)0)
#define PROFILE_DRAW()         ((void)0)
#define PROFILE_CALL(a,b)      (a)
#define PROFILE_NEXT(a,b)      (a)

#endif

// ======================================================================

#endif
